#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#define MYPORT 3490
#define BACKLOG 10

int main(void)
{
    int sockfd, new_fd;
    struct sockaddr_in my_addr;
    struct sockaddr_in their_addr;
    int sin_size;
    char host[ NI_MAXHOST ];

    if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
        perror("socket");
        exit(1);
    }

    my_addr.sin_family = AF_INET; 
    my_addr.sin_port = htons(MYPORT);
    my_addr.sin_addr.s_addr = INADDR_ANY;
    memset(&(my_addr.sin_zero), '\0', 8);

    // lose the pesky "address already in use" error message
    {
      int yes=1;
      if (setsockopt(sockfd , SOL_SOCKET, SO_REUSEADDR, &yes,
		     sizeof(int)) == -1) {
	perror("setsockopt");
	exit(1);
      }
    }
    


    if (bind(sockfd, (struct sockaddr *)&my_addr, 
	     sizeof(struct sockaddr)) == -1) {
        perror("bind");
        exit(1);
    }

    if (listen(sockfd, BACKLOG) == -1) {
        perror("listen");
        exit(1);
    }

    while(1) {  // main accept() loop
        sin_size = sizeof(struct sockaddr_in);
        if ((new_fd = accept(sockfd, 
			     (struct sockaddr *)&their_addr, 
			     &sin_size)) == -1) {
            perror("accept");
            continue;
        }

	/* Find out the client host name */
	if ( getnameinfo( ( struct sockaddr * ) &their_addr,
			  sizeof( their_addr ),
			  host, NI_MAXHOST, NULL, 0, 0) != 0)
	  {
	    perror( "getnameinfo" );
	    strncpy( host, "<UNKNOW_ERROR>", NI_MAXHOST - 1 );
	  }
	  
	printf( "server: got connection from %s (%s)\n", 
		inet_ntoa( their_addr.sin_addr ),
		host );
	

        if (send(new_fd, "Hello, world!\n", 14, 0) == -1)
	  perror("send");
	close(new_fd);
    }
    return 0;
}

/* author: Gustavo Sverzut Barbieri (http://www.gustavobarbieri.com.br) */
