import java.rmi.Remote;
import java.rmi.RemoteException;

public interface FileServerInterface extends Remote
{
    /**
     * show credits
     *
     * @return credits
     */
    public String cred()
	throws RemoteException;



    /**
     * get file chunck from server.
     *
     * @param offset chunck init pos
     * @param count  how many bytes to read (maximum)
     * 
     * @return status=0 on EOF, status=-1 on Failure, status=1 on success.
     *         If failure, exception is kept in e, otherwise count bytes are
     *         stored in buffer.
     */
    public GetFileResult getFile( String name, long offset, int count )
	throws RemoteException;



    /**
     * list dir contents
     *
     * @param name directory name/path
     *
     * @return list of strings
     */
    public String[] list( String name )
	throws RemoteException;



    /**
     * test file existence of given file
     * 
     * @param name file to test
     * 
     * @return true if exists, false otherwise
     */
    public boolean exists( String name )
	throws RemoteException;    
}
