import java.io.*;
import java.rmi.*;

public class FileClient 
{
    static int BLOCK_SIZE     = 8000;
    FileServerInterface fs;

    public FileClient( String host )
    {

	if ( System.getSecurityManager() == null ) 
	    System.setSecurityManager( new RMISecurityManager() );

        try {
            String name = "//" + host + "/FileServer";
            fs = (FileServerInterface) Naming.lookup( name );
        } catch (Exception e) {
            System.err.println( "Exception: " + e );
        }	
    }


    public void cred()
    {
	while ( true )
	    {
		try {
		    String cred = fs.cred();
		    System.out.println( "CRED: " + cred );
		    break;
		} catch ( RemoteException e ) {
		    System.err.println( "Connection Error: " + e );
		    System.err.println( "Trying Again..." );
		}
	    }
    }



    public void getFile( String src, String dst, boolean statistics )
    {
	long offset = 0;
	GetFileResult gr;
	FileOutputStream file;
	long count = 0;
	long t0, t1, t2;
	
	try {
	    file = new FileOutputStream( dst );
	} catch ( FileNotFoundException e ) {
	    System.err.println( "ERROR: " + e );
	    return;
	}

	t0 = System.currentTimeMillis();
	while ( true )
	    {
		while ( true )
		    {
			try {
			    t1 = System.currentTimeMillis();
			    gr = fs.getFile( src, offset, BLOCK_SIZE );
			    count += gr.count;
			    if ( statistics )
				{
				    t2 = System.currentTimeMillis();
				    System.out.println( "STAT: Got " + 
							gr.count + 
							" bytes in "+
							(t2 - t1) + " ms." );
				}

			    break;
			} catch ( RemoteException e ) {
			    System.err.println( "Connection Error: " + e );
			    System.err.println( "Trying Again..." );
			}
		    }


		    
		if ( gr.status >= 0 )
		    {
			offset += gr.buffer.length;
			try {			    
			    file.write( gr.buffer, 0, gr.count );
			    if ( gr.status == 0 )
				break;

			} catch ( IOException e ) {
			    System.err.println( "ERROR: " + e );
			    break;
			}
		    }
		else
		    {
			System.err.println( "ERROR: " + gr.e );
			break;
		    }
	    }

	t2 = System.currentTimeMillis();
	String unity;
	double rate;
	if ( count > 1024 * 1024 )
	    {
		count /= ( 1024 * 1024 );
		unity = " Mb";
	    }
	else if ( count > 1024 )
	    {
		count /= 1024;
		unity = " Kb";
	    }
	else
	    unity = " b";

	rate = (float)count * 1000.0 / (t2 - t0);

	System.out.println( "Got '" + src + "' " +
			    count + unity + " in " +
			    (t2 - t0) + "ms  [" + rate + unity + "/s]" );
	

	try {
	    file.close();
	} catch ( IOException e ) {
	    System.err.println( "ERROR: " + e );
	} 
    }

    public void list( String name )
    {
	int i;
	String dir[];
	while ( true )
	    {
		try {
		    dir = fs.list( name );
		    break;
		} catch ( RemoteException e ) {
		    System.err.println( "Connection Error: " + e );
		    System.err.println( "Trying Again..." );
		}
	    }

	if ( dir == null )
	    System.err.println( "Could not list dir '" + name + "'" );
	else
	    for ( i=0; i < dir.length; i++ )
		System.out.println( dir[ i ] );
    }

    public void exists( String name )
    {
	String result = new String();

	boolean r;
	while ( true )
	    {
		try {
		    r =  fs.exists( name );
		    break;
		} catch ( RemoteException e ) {
		    System.err.println( "Connection Error: " + e );
		    System.err.println( "Trying Again..." );
		}
	    }
	result = "File '" + name + "' does ";
	if ( ! r )
	    result += "NOT ";
	
	result += "exist.";

	System.out.println( result );
    }


    public static String getLine() {
	String inputLine = "";
	
	
	return inputLine;
    }


    public static void main( String[] args ) {

	if ( args.length < 1 )
	    {
		System.err.println( "You must provide the server name!" );
		return;
	    }

	FileClient fc = new FileClient( args[ 0 ] );

	BufferedReader in = new BufferedReader( new InputStreamReader( System.in ), 1 );

	String line = "";
	String a[];
	while ( true )
	    {
		System.out.print( "> " );
		try {
		    line = in.readLine();
		} catch ( IOException e ) {
		    System.err.println( "Exception: " + e );
		    continue;
		}
		line.trim();
		a = line.split( " " );

		if ( a.length < 1 )
		    continue;
		
		if ( ( a[ 0 ].compareTo( "quit" ) == 0 ) ||
		     ( a[ 0 ].compareTo( "exit" ) == 0 ) )
		    break;
		else if ( a[ 0 ].compareTo( "list" ) == 0 )
		    {
			String dir = ".";
			if ( a.length > 1 )
			    dir = a[ 1 ];

			fc.list( dir );
		    }
		else if ( a[ 0 ].compareTo( "exist" ) == 0 )
		    {
			if ( a.length < 2 )
			    {
				System.out.println( "Filename required!" );
				continue;
			    }
			fc.exists( a[ 1 ] );
		    }
		else if ( a[ 0 ].compareTo( "get" ) == 0 )
		    {
			String src;
			String dst;
			if ( a.length == 2 )
			    {
				src = a[ 1 ];
				dst = a[ 1 ];
			    }
			else if ( a.length > 2 )
			    {
				src = a[ 1 ];
				dst = a[ 2 ];
			    }
			else
			    {
				System.out.println("Missing source filename!");
				continue;
			    }
			fc.getFile( src, dst, false );
		    }
		else if ( a[ 0 ].compareTo( "getstat" ) == 0 )
		    {
			String src;
			String dst;
			if ( a.length == 2 )
			    {
				src = a[ 1 ];
				dst = a[ 1 ];
			    }
			else if ( a.length > 2 )
			    {
				src = a[ 1 ];
				dst = a[ 2 ];
			    }
			else
			    {
				System.out.println("Missing source filename!");
				continue;
			    }
			fc.getFile( src, dst, true );
		    }
		else if ( a[ 0 ].compareTo( "cred" ) == 0 )
		    fc.cred();
		
	    }
	
    }
}
