import java.io.*;
import java.net.*;
import java.rmi.*;
import java.rmi.server.*;

public class FileServer 
    extends UnicastRemoteObject 
    implements FileServerInterface 
{
    public String cwd;

    /**
     * show credits
     *
     * @return credits
     */
    public String cred()
	throws RemoteException
    {
	return new String("Gustavo Sverzut Barbieri <ra008849@ic.unicamp.br>");
    }

    /**
     * get file chunck from server.
     *
     * @param offset chunck init pos
     * @param count  how many bytes to read (maximum)
     * 
     * @return status=0 on EOF, status=-1 on Failure, status=1 on success.
     *         If failure, exception is kept in e, otherwise count bytes are
     *         stored in buffer.
     */
    public GetFileResult getFile( String name, long offset, int count )
	throws RemoteException
    {
	GetFileResult gr = new GetFileResult( count );
	try {
	    int i;
	    FileInputStream file = new FileInputStream( new File( cwd, name ));
	    
	    file.skip( offset );
	    // JAVA SUX !!! 
	    // Implement it this way or loose a year reading the docs...
	    gr.count = 0;
	    while ( true )
		{
		    i = file.read();
		    if ( i == -1 )
			{
			    gr.status =  0;
			    break;
			}
		    else
			{
			    gr.status = 1;
			    gr.buffer[ gr.count ] = (byte)i;
			    gr.count ++;
			}
		    if ( gr.count >= gr.buffer.length )
			break;
		}
	    file.close();
	} 
	catch( Exception e ) {
	    gr.status = -1;
	    gr.e      = e;
	}
	
	return gr;
    }


    /**
     * list dir contents
     *
     * @param name directory name/path
     *
     * @return list of strings
     */
    public String[] list( String name )
	throws RemoteException
    {
	try {
	    File dir = new File( cwd, name );
	    return dir.list();
	} catch ( Exception e ) {
	    System.out.println( "Exception: " + e );
	    return null;
	}
    }



    /**
     * test file existence of given file
     * 
     * @param name file to test
     * 
     * @return true if exists, false otherwise
     */
    public boolean exists( String name )
	throws RemoteException
    {
	File file = new File( cwd, name );
	return file.exists();
    }


    public FileServer() throws RemoteException {
	super();
	cwd = System.getProperty( "user.dir" );
    }



    public static void main( String args[] )
    {
	String hostname;
	String name;

	if ( System.getSecurityManager() == null ) 
	    System.setSecurityManager( new RMISecurityManager() );

	try {
	    hostname = InetAddress.getLocalHost().getHostName();
	} catch ( java.net.UnknownHostException e ) {
	    hostname = "localhost";
	}

        name = "//" + hostname + "/FileServer";
        try {
            FileServer fs = new FileServer();
            Naming.rebind( name, fs );
            System.err.println( "Server Launched." );

        } catch ( Exception e ) 
	    {
		System.err.println( "ComputeEngine exception: " + 
				    e.getMessage() );
		e.printStackTrace();
	    }       
    }
}


